;-------------------------------------------
;
; ATR2CAR starter for SWITCHABLE XEGS
; CARTRIDGE with 128 sectors
; (c) 2022 GienekP
;
;-------------------------------------------

RAMPROC = $0128
OFFSET  = RAMPROC-ZEROCP

;-------------------------------------------

TMP     = $32

;-------------------------------------------

CRITIC  = $42
RAMTOP  = $6A

DMACTLS = $022F
PDVMSK  = $0247
MEMTOP  = $02E5
DSCTLN  = $02D5
DVSTAT  = $02ea
DDEVIC  = $0300
DUNIT   = $0301
DCMND   = $0302
DSTATS  = $0303
DBUFA   = $0304
DBYT    = $0308
DAUX1   = $030A
DAUX2   = $030B
BASICF  = $03F8
GINTLK  = $03FA

TRIG3   = $D013
IRQEN   = $D20E
IRQST   = $D20E
PORTB   = $D301
DMACTL  = $D400
VCOUNT  = $D40B
NMIEN   = $D40E
WSYNC   = $D40A
WAIT    = $C0DF
RESETWM = $C290
RESETCD = $C2C8
BOOT    = $C58B
JDSKINT = $E453
JSIOINT = $E459

;-------------------------------------------
; SWITCHABLE XEGS CARTRIDGE

        OPT h-f+

        ORG $BC00

;-------------------------------------------
; INITCART ROUTINE

INIT    rts

;-------------------------------------------
; CARTRUN ROUTINE

BEGIN   jsr IRQDIS
        jsr ROM2RAM
        jsr SETRAM
        jsr OVRDINT
        lda #$4c ;Zamiana jmp ($0004) na ROMON
        sta $c638
        lda #<[ROMON+OFFSET]
        sta $c638+1
        lda #>[ROMON+OFFSET]
        sta $c638+2
        jsr RESERVE
        jsr IRQENB
        jsr FINAL
        ldx #$FF
        LDA #$0C
        STA $0342
        lda #$c4    ; adres JMP ($000A) DOSVEC
        pha
        lda #$25
        pha
        jmp RAMPROC+GOBOOT-ZEROCP

;-------------------------------------------
; IRQ ENABLE

IRQENB  lda #$40
        sta NMIEN
        lda #$F7
        sta IRQST
;        lda DMACTLS
;        sta DMACTL
        cli
        rts

;-------------------------------------------
; IRQ DISABLE

IRQDIS  sei
        lda #$00
;        sta DMACTL
        sta NMIEN
        sta IRQEN
        sta IRQST
        rts

;-------------------------------------------
; COPY ROM TO RAM

ROM2RAM ldy #$00
        sty TMP
        lda #$C0
L1      sta TMP+1
L2      lda (TMP),Y
        dec PORTB
        sta (TMP),Y
        inc PORTB
        iny
        bne L2
        inc TMP+1
        beq L3
        lda TMP+1
        cmp #$D0
        bne L2
        lda #$D8
        bne L1
L3      rts

;-------------------------------------------
; SET RAM & DISABLE BASIC

SETRAM  lda #$FE
        sta PORTB
        lda #$01
        sta BASICF
        rts

;-------------------------------------------
; COPY NEW SIOINT PROCEDURE

OVRDINT lda #<[RAMPROC] ; patch na $E459 w ROM
        ldy #>[RAMPROC]
        sta $E45A
        sty $E45B
        rts

;-------------------------------------------
; COPY TO $RAMPROC FOR "KILLERS" PORTB

RESERVE ldy #ZEROEND-ZEROCP-1
RESCPY  lda ZEROCP,Y
        sta RAMPROC,Y
        dey
        bpl RESCPY
        rts

;-------------------------------------------
; FINAL VALUES

FINAL   lda #$1F
        sta MEMTOP
        lda #$BC
        sta MEMTOP+1
        lda #$C0
        sta RAMTOP
        lda #$01
        sta PDVMSK
VCL1    lda VCOUNT
        cmp #$3
        bne VCL1
TSTMAX  lda VCOUNT
        cmp #$8A
        bne VCL2
        sta RAMPROC+LICNT-ZEROCP+1
VCL2    cmp #$00
        bne TSTMAX
        rts

;-------------------------------------------
; RELOC CODE FOR RAMPROC

ZEROCP  LDA #>[AROUND-1]
LI2     PHA
        PHA
ZERO    lda VCOUNT
LICNT   eor #$72    ; $72->NTSC $8A->PAL
        bne ZERO
        sta $D500
        rts

E453    LDA #>[AROUND2-1]
        bne LI2

CPYSEC  lda #$00    ; $010F
        sta $D500
KMP1    lda $8000,Y
        stx $D500
KMP2    sta $0400,Y
        iny
        bpl CPYSEC
        ldy #$01

BACK    cli
        stx $D500
        inx
        rts

ROMON   ldy #$FF
        sty PORTB
        jmp ($0004)

GOBOOT  stx $D500
        inx
        stx GINTLK
        JSR $E456
        LDA #$03
        STA $0342,X
        LDA #EDYT&$FF
        STA $0344,X
        LDA #EDYT/256
        STA $0345,X
        LDA #$0C
        STA $034A,X
        JSR $E456
        lda #>[$c413-1]
        pha
        lda #<[$c413-1]
        pha
        jmp BOOT

ZEROEND

EDYT    = $C448     ; E:,$9B

;-------------------------------------------

; AROUND SIO INTerface

        ORG $BDBE

AROUND  sei
        clc
        lda DDEVIC
        adc DUNIT
        cmp #$32
        bne GO_CIO
        lda DCMND
AR1     ldx #$FF
        cmp #$52
        beq SECREAD
        cmp #$57
        beq STATOK
        cmp #$50
        beq STATOK
        cmp #$53
        bne UNKWCMD
        lda DBUFA   ; status dla $E459
        sec
        sbc #$7C
        sta KMP2+OFFSET+1
        lda DBUFA+1
        sbc #$00
        sta KMP2+OFFSET+2
        ldy #$7C
        lda <[STATUS-$7C]
        sta KMP1+OFFSET+1
        lda >[STATUS-$7C]
        sta KMP1+OFFSET+2
        jmp RAMPROC+CPYSEC-ZEROCP
STATOK  ldy #$01
        sty DSTATS
UNKWCMD jmp RAMPROC+BACK-ZEROCP

GO_CIO  lda >[JSIOINT-1]    ; $E459
        pha
        lda <[JSIOINT-1]
        pha
        ldx #$FF
        jmp RAMPROC+BACK-ZEROCP

SECREAD lda DAUX2
        asl
        asl
        and #$7F
        sta TMP
        lda DAUX1
        lsr
        and #$1F
        ora #$80
        sta KMP1+OFFSET+2
        lda DAUX1
        lsr
        lsr
        lsr
        lsr
        lsr
        lsr
        ora TMP
        sta RAMPROC+CPYSEC-ZEROCP+1 ; bank sektora LDA #$FF
        ldy #$01
        sty DSTATS
        dey
        lda DAUX1
        and #$01
        beq NOHALF
        lda #$80
NOHALF  sta KMP1+OFFSET+1
        lda DBUFA
        sta KMP2+OFFSET+1
        lda DBUFA+1
        sta KMP2+OFFSET+2
        ldx #$FF
        stx WSYNC
        jmp RAMPROC+CPYSEC-ZEROCP

;-------------------------------------------

        ORG $BEBF

AROUND2 sei
        clc
        lda DDEVIC
        adc DUNIT
        cmp #$32
        bne GO_CIO2
        lda #$31
        sta DDEVIC
        lda DCMND     ; status dla $E453
        cmp #$53
        bne AR3
        lda #<DVSTAT  ; $ea
        sta DBUFA
        lda #>DVSTAT  ; $02
        sta DBUFA+1
        ldx #$03
AR4     lda STATUS,x
        sta DVSTAT,x
        dex
        bpl AR4
        ldy #$01
        sty DSTATS
;       ldx #$FF
        jmp RAMPROC+BACK-ZEROCP
AR3     ldy DSCTLN
        sty DBYT
        ldy DSCTLN+1
        sty DBYT+1
        jmp AR1

GO_CIO2 lda >[JDSKINT-1]    ; $E453
        pha
        lda <[JDSKINT-1]
        pha
        ldx #$FF
        jmp RAMPROC+BACK-ZEROCP

        ORG $BFF0

STATUS  dta $98,$ff,$01,$00

        ORG $BFFA
        dta <BEGIN, >BEGIN, $00, $04, <INIT, >INIT

;-------------------------------------------
